#ifndef LANDSCAPETOPOLOGY_HH_
#define LANDSCAPETOPOLOGY_HH_

#include "ell/State.hh"

#include <string>

namespace ell {

/**
 * An abstract representation of the landscapes topology that stores minima and
 * allows for saddle point storage.
 * 
 * @author Hannes Kochni, Martin Mann
 */
class LandscapeTopology {
	
protected:
	
	//! Comparison function that compares on energy and string representation
	//! @return true if energy of a smaller than b or if equal if the string 
	//!         represenation of a is lexicographically smaller than b
	static
	bool isSmaller_E_S(const State* a, const State* b);


	
public:
	
	//! Index value that is used to propagate the information about invalid
	//! indexing.
	static const size_t INVALID_INDEX;
	
	//! Destruction
	virtual 
	~LandscapeTopology();
	
	//! Sets a saddle point that connects two minima 
	//! @param i index of the first minimum to connect
	//! @param j index of the second minimum to connect
	//! @param s the State representating the saddle point
	//! @return whether or not the data structure has been changed
	virtual 
	bool 
	addSaddle(const size_t i, const size_t j, const State* const s) = 0;
	
	//! Sets a saddle point that connects two minima 
	//! @param m_i the first minimum to connect
	//! @param m_j the second minimum to connect
	//! @param s the State representating the saddle point
	//! @return whether or not the data structure has been changed
	virtual 
	bool 
	addSaddle(const State* const m_i, const State* const m_j, const State* const s) = 0;
	
	//! Adds an UNKNOWN minimum to the topology.
	//! NOTE : this operation should NOT change the indices of already existing
	//!        minima in the topology !
	//! FURTHER : the minimum is not checked if it is already known
	//! @param m the first minimum to add
	//! @return the index of the minimum in the landscape
	virtual 
	const size_t 
	addMin(const State* const m) = 0;
	
	//! Access to the minimum index.
	//! @param m the minimum of that the index should be accessed
	//! @return  the index of the minimum, or INVALID_INDEX if 'm' is no known 
	//!          minimum
	virtual 
	const size_t 
	getMinIndex(const State* const m) const = 0;
	
	//! Access to the number of stored minima.
	//! @return the number of minima
	virtual 
	const size_t 
	getMinCount() const = 0;
	
	//! Access to a minimum.
	//! @param i index of the minimum to access
	//! @return the State with the specified minimum index
	virtual 
	const State* const 
	getMin(size_t i) const = 0;
	
	//! Access to the global minimum of the landscape with minimal free energy
	//! (mfe).
	//! @return the minimum with smallest energy (and smallest string 
	//!         representation for tiebreaking)
	virtual 
	const State* const 
	getMFEState() const = 0;

	//! Calculates the root mean square deviation between landscape topologies.
	//! @param lt the landscape topology to calculate the RMSD
	//! @return the RMSD
	virtual 
	double 
	getRMSD(const LandscapeTopology* const lt) const = 0;
	
	//! Sorts the minima ascending by their energy (mfe minimum gets index 0). 
	//! If energy is equal they are sorted by their string representation.
	virtual 
	void 
	sort() = 0;

	//! Check if the minima of the landscape are sorted ascendingly by their 
	//! energy (mfe minimum gets index 0). 
	//! If energy is equal they are sorted by their string representation.
	virtual 
	bool 
	isSorted() const = 0;

	//! Clears the data structure and removes all stored information to be 
	//! filled again afterwards. 
	virtual 
	void 
	clear() = 0;

	//! Writes the current landscape topology to stream.
	//! @param out the output stream to write to.
	//! @param StateDescription a string that describes the type of states 
	//!        written to the stream (will be part of the LT information header)
	virtual 
	void
	write( std::ostream & out, const std::string & StateDescription ) const = 0;

	//! Fills the landscape topology from stream.
	//! @param in the input stream to read from.
	//! @param templateState the State template to create new State objects via
	//!        its 'fromString' function
	//! @param StateDescription a string that describes the type of states 
	//!        that should be read and that is compared to the LT information 
	//!        header in the stream
	//! @return an error encoding and an error string :
	//!          0 = no error occured, read was successfull
	//!         -1 = landscape topology type not supported
	//!         -2 = state description differs from the given one
	//!         -3 = other read errors
	virtual 
	std::pair<int,std::string>
	read(	std::istream & in, 
			const State * const templateState, 
			const std::string & StateDescription ) = 0;

};

}// ell


#endif /*LANDSCAPETOPOLOGY_HH_*/
